@@ -730,6 +730,31 @@ class DBAccess():
         logger.error('Error getting expid from details')
         return None
 
+    def get_term_family_graph(self, terms, relation='both'):
+        '''Get the ontology parents/children graph for a set of terms
+
+        Parameters
+        ----------
+        terms: list of str
+            the terms to get the tree for
+        term_type: str
+            'both' to get parents and children
+            'parent' to get only parents
+            'child' to get only children
+
+        Returns
+        -------
+        graph in networkx json format (node_link_data)
+        '''
+        rdata = {}
+        rdata['terms'] = terms
+        res = self._get('ontology/get_family_graph', rdata, param_json=True)
+        if res.status_code == 200:
+            graph = res.json()['family']
+            return graph
+        logger.error('Error getting get_term_family_graph.')
+        return None
+
     def get_experiment_info(self, expid):
         """
         get the information about a given experiment dataid
@@ -932,6 +957,12 @@ class DBAccess():
             the dbbact assigned taxonomy for each sequence
         '''
         logger.info('Getting dbBact annotations for %d sequences, please wait...' % len(sequences))
+        if max_id is not None:
+            logger.info('Limited to maximal annotationid: %d' % max_id)
+        if get_parents:
+            logger.debug('Getting also parents')
+        if get_term_info:
+            logger.debug('Getting also term_info')
         rdata = {}
         rdata['sequences'] = list(sequences)
         rdata['get_term_info'] = get_term_info
@@ -1306,6 +1337,8 @@ class DBAccess():
         g2_features = np.array(sorted(list(g2_features)))
         exp_features = np.hstack([g1_features, g2_features])
 
+        logger.debug('using %s features for group1, %d features for group2' % (len(g1_features), len(g2_features)))
+
         # filter keeping only annotations containing focus_terms (if not None)
         if focus_terms is not None:
             logger.debug('filtering annotations for %d focus terms.' % len(focus_terms))
@@ -1332,7 +1365,31 @@ class DBAccess():
             feature_terms = self._get_all_term_counts(exp_features, seq_annotations, all_annotations, ignore_exp=ignore_exp, score_method=score_method, use_term_pairs=use_term_pairs)
         elif term_type == 'parentterm':
             # score for each term including the parent terms (based on the ontology structure) for each term
-            raise ValueError('parentterm not supported yet')
+            # make sure we have the ontology parents
+            annotation_iterator = iter(all_annotations.values())
+            first_annotation = next(annotation_iterator)
+            if 'parents' not in first_annotation:
+                raise ValueError('Annotations do not seem to contain parent terms. sample annotation:\n%s' % first_annotation)
+
+            # let's add all parent terms to annotations
+            logger.debug('adding parent terms')
+            for cannotation_id, cannotation in all_annotations.items():
+                details = cannotation['details']
+                old_details_terms = set([x[1] for x in details])
+                for cdir, cterms in cannotation['parents'].items():
+                    for cpterm in cterms:
+                        if len(cpterm) == 0:
+                            continue
+                        if cpterm in old_details_terms:
+                            continue
+                        details.append([cdir, cpterm, 'parent'])
+                        old_details_terms.add(cpterm)
+                cannotation['details'] = details
+                all_annotations[cannotation_id] = cannotation
+
+            # score for each term
+            logger.debug('getting "terms" per feature')
+            feature_terms = self._get_all_term_counts(exp_features, seq_annotations, all_annotations, ignore_exp=ignore_exp, score_method=score_method, use_term_pairs=use_term_pairs)
         elif term_type == 'annotation':
             # score for each annotation
             logger.debug('getting "annotations" per feature')
@@ -1392,6 +1449,9 @@ class DBAccess():
         new_term_list = []
         orig_term_list = term_list.copy()
         for cterm in term_list:
+            if len(cterm) == 0:
+                print('EMPTY????')
+                cterm = 'EMPTY???'
             if cterm[0] == '-':
                 ccterm = 'LOWER IN ' + cterm[1:]
                 cterm = cterm[1:]
